from collections import OrderedDict
try:
    from unittest import mock
except ImportError:
    import mock

import pytest

from graphql_ws import base, constants


@pytest.fixture
def cc():
    cc = base.BaseConnectionContext(ws=None)
    cc.operations = {"yes": "1"}
    return cc


@pytest.fixture
def ss():
    return base.BaseSubscriptionServer(schema=None)


class TestConnectionContextOperation:
    def test_no_operations_initially(self):
        cc = base.BaseConnectionContext(ws=None)
        assert not cc.operations

    def test_has_operation(self, cc):
        assert cc.has_operation("yes")

    def test_has_operation_missing(self, cc):
        assert not cc.has_operation("no")

    def test_register_operation(self, cc):
        cc.register_operation("new", "2")
        assert "new" in cc.operations

    def test_get_operation(self, cc):
        assert cc.get_operation("yes") == "1"

    def test_remove_operation(self, cc):
        cc.remove_operation("yes")
        assert not cc.operations


class TestConnectionContextNotImplentedMethods:
    def test_receive(self):
        with pytest.raises(NotImplementedError):
            base.BaseConnectionContext(ws=None).receive()

    def test_send(self):
        with pytest.raises(NotImplementedError):
            base.BaseConnectionContext(ws=None).send("TEST")

    def test_closed(self):
        with pytest.raises(NotImplementedError):
            base.BaseConnectionContext(ws=None).closed

    def test_close(self):
        with pytest.raises(NotImplementedError):
            base.BaseConnectionContext(ws=None).close(123)


class TestProcessMessage:
    def test_init(self, ss, cc):
        ss.on_connection_init = mock.Mock()
        ss.process_message(
            cc, {"id": "1", "type": constants.GQL_CONNECTION_INIT, "payload": "payload"}
        )
        ss.on_connection_init.assert_called_with(cc, "1", "payload")

    def test_terminate(self, ss, cc):
        ss.on_connection_terminate = mock.Mock()
        ss.process_message(cc, {"id": "1", "type": constants.GQL_CONNECTION_TERMINATE})
        ss.on_connection_terminate.assert_called_with(cc, "1")

    def test_start(self, ss, cc):
        ss.get_graphql_params = mock.Mock()
        ss.get_graphql_params.return_value = {"params": True}
        cc.has_operation = mock.Mock()
        cc.has_operation.return_value = False
        ss.unsubscribe = mock.Mock()
        ss.on_start = mock.Mock()
        ss.process_message(
            cc, {"id": "1", "type": constants.GQL_START, "payload": {"a": "b"}}
        )
        assert not ss.unsubscribe.called
        ss.on_start.assert_called_with(cc, "1", {"params": True})

    def test_start_existing_op(self, ss, cc):
        ss.get_graphql_params = mock.Mock()
        ss.get_graphql_params.return_value = {"params": True}
        cc.has_operation = mock.Mock()
        cc.has_operation.return_value = True
        ss.unsubscribe = mock.Mock()
        ss.on_start = mock.Mock()
        ss.process_message(
            cc, {"id": "1", "type": constants.GQL_START, "payload": {"a": "b"}}
        )
        assert ss.unsubscribe.called
        ss.on_start.assert_called_with(cc, "1", {"params": True})

    def test_start_bad_graphql_params(self, ss, cc):
        ss.get_graphql_params = mock.Mock()
        ss.get_graphql_params.return_value = None
        cc.has_operation = mock.Mock()
        cc.has_operation.return_value = False
        ss.send_error = mock.Mock()
        ss.unsubscribe = mock.Mock()
        ss.on_start = mock.Mock()
        ss.process_message(
            cc, {"id": "1", "type": constants.GQL_START, "payload": {"a": "b"}}
        )
        assert ss.send_error.called
        assert ss.send_error.call_args[0][:2] == (cc, "1")
        assert isinstance(ss.send_error.call_args[0][2], Exception)
        assert not ss.on_start.called

    def test_stop(self, ss, cc):
        ss.on_stop = mock.Mock()
        ss.process_message(cc, {"id": "1", "type": constants.GQL_STOP})
        ss.on_stop.assert_called_with(cc, "1")

    def test_invalid(self, ss, cc):
        ss.send_error = mock.Mock()
        ss.process_message(cc, {"id": "1", "type": "unknown"})
        assert ss.send_error.called
        assert ss.send_error.call_args[0][:2] == (cc, "1")
        assert isinstance(ss.send_error.call_args[0][2], Exception)


def test_get_graphql_params(ss, cc):
    payload = {
        "query": "req",
        "variables": "vars",
        "operationName": "query",
        "context": "ctx",
    }
    assert ss.get_graphql_params(cc, payload) == {
        "request_string": "req",
        "variable_values": "vars",
        "operation_name": "query",
        "context_value": "ctx",
    }


def test_build_message(ss):
    assert ss.build_message("1", "query", "PAYLOAD") == {
        "id": "1",
        "type": "query",
        "payload": "PAYLOAD",
    }


def test_build_message_partial(ss):
    assert ss.build_message(id="1", op_type=None, payload=None) == {"id": "1"}
    assert ss.build_message(id=None, op_type="query", payload=None) == {"type": "query"}
    assert ss.build_message(id=None, op_type=None, payload="PAYLOAD") == {
        "payload": "PAYLOAD"
    }
    assert ss.build_message(id=None, op_type=None, payload=None) == {}


def test_send_execution_result(ss):
    ss.execution_result_to_dict = mock.Mock()
    ss.execution_result_to_dict.return_value = {"res": "ult"}
    ss.send_message = mock.Mock()
    ss.send_message.return_value = "returned"
    assert "returned" == ss.send_execution_result(cc, "1", "result")
    ss.send_message.assert_called_with(cc, "1", constants.GQL_DATA, {"res": "ult"})


def test_execution_result_to_dict(ss):
    result = mock.Mock()
    result.data = "DATA"
    result.errors = "ER"
    result_dict = ss.execution_result_to_dict(result)
    assert isinstance(result_dict, OrderedDict)
    assert result_dict == {
        "data": "DATA",
        "errors": [{"message": "E"}, {"message": "R"}],
    }


def test_send_message(ss, cc):
    ss.build_message = mock.Mock()
    ss.build_message.return_value = {"mess": "age"}
    cc.send = mock.Mock()
    cc.send.return_value = "returned"
    assert "returned" == ss.send_message(cc)
    cc.send.assert_called_with('{"mess": "age"}')


class TestSSNotImplemented:
    def test_handle(self, ss):
        with pytest.raises(NotImplementedError):
            ss.handle(ws=None, request_context=None)

    def test_on_open(self, ss):
        with pytest.raises(NotImplementedError):
            ss.on_open(connection_context=None)

    def test_on_connect(self, ss):
        with pytest.raises(NotImplementedError):
            ss.on_connect(connection_context=None, payload=None)

    def test_on_close(self, ss):
        with pytest.raises(NotImplementedError):
            ss.on_close(connection_context=None)

    def test_on_connection_init(self, ss):
        with pytest.raises(NotImplementedError):
            ss.on_connection_init(connection_context=None, op_id=None, payload=None)

    def test_on_stop(self, ss):
        with pytest.raises(NotImplementedError):
            ss.on_stop(connection_context=None, op_id=None)

    def test_on_start(self, ss):
        with pytest.raises(NotImplementedError):
            ss.on_start(connection_context=None, op_id=None, params=None)



import sys

if sys.version_info > (3,):
    collect_ignore = ["test_django_channels.py"]
    if sys.version_info < (3, 6):
        collect_ignore.append('test_gevent.py')
else:
    collect_ignore = ["test_aiohttp.py"]



import asyncio
import pickle

import aredis


class AsyncioPubsub:

    def __init__(self):
        self.subscriptions = {}
        self.sub_id = 0

    async def publish(self, channel, payload):
        if channel in self.subscriptions:
            for q in self.subscriptions[channel].values():
                await q.put(payload)

    def subscribe_to_channel(self, channel):
        self.sub_id += 1
        q = asyncio.Queue()
        if channel in self.subscriptions:
            self.subscriptions[channel][self.sub_id] = q
        else:
            self.subscriptions[channel] = {self.sub_id: q}
        return self.sub_id, q

    def unsubscribe(self, channel, sub_id):
        if sub_id in self.subscriptions.get(channel, {}):
            del self.subscriptions[channel][sub_id]
        if not self.subscriptions[channel]:
            del self.subscriptions[channel]


class AsyncioRedisPubsub:

    def __init__(self, host='localhost', port=6379, *args, **kwargs):
        self.redis = aredis.StrictRedis(host, port, *args, **kwargs)
        self.pubsub = self.redis.pubsub(ignore_subscribe_messages=True)
        self.subscriptions = {}
        self.sub_id = 0
        self.task = None

    async def publish(self, channel, payload):
        await self.redis.publish(channel, pickle.dumps(payload))

    async def subscribe_to_channel(self, channel):
        self.sub_id += 1
        q = asyncio.Queue()
        if channel in self.subscriptions:
            self.subscriptions[channel][self.sub_id] = q
        else:
            await self.pubsub.subscribe(channel)
            self.subscriptions[channel] = {self.sub_id: q}
            if not self.task:
                self.task = asyncio.ensure_future(
                    self._wait_and_get_messages())
        return self.sub_id, q

    async def unsubscribe(self, channel, sub_id):
        if sub_id in self.subscriptions.get(channel, {}):
            del self.subscriptions[channel][sub_id]
        if not self.subscriptions[channel]:
            await self.pubsub.unsubscribe(channel)
            del self.subscriptions[channel]
        if not self.subscriptions:
            self.task.cancel()

    async def _wait_and_get_messages(self):
        while True:
            msg = await self.pubsub.get_message()
            if msg:
                channel = msg['channel'].decode()
                if channel in self.subscriptions:
                    for q in self.subscriptions[channel].values():
                        await q.put(pickle.loads(msg['data']))
            await asyncio.sleep(.001)



from inspect import isawaitable
from asyncio import ensure_future, wait, shield

from aiohttp import WSMsgType
from graphql.execution.executors.asyncio import AsyncioExecutor

from .base import (
    ConnectionClosedException, BaseConnectionContext, BaseSubscriptionServer)
from .observable_aiter import setup_observable_extension

from .constants import (
    GQL_CONNECTION_ACK,
    GQL_CONNECTION_ERROR,
    GQL_COMPLETE
)

setup_observable_extension()


class AiohttpConnectionContext(BaseConnectionContext):
    async def receive(self):
        msg = await self.ws.receive()
        if msg.type == WSMsgType.TEXT:
            return msg.data
        elif msg.type == WSMsgType.ERROR:
            raise ConnectionClosedException()
        elif msg.type == WSMsgType.CLOSING:
            raise ConnectionClosedException()
        elif msg.type == WSMsgType.CLOSED:
            raise ConnectionClosedException()

    async def send(self, data):
        if self.closed:
            return
        await self.ws.send_str(data)

    @property
    def closed(self):
        return self.ws.closed

    async def close(self, code):
        await self.ws.close(code=code)


class AiohttpSubscriptionServer(BaseSubscriptionServer):
    def __init__(self, schema, keep_alive=True, loop=None):
        self.loop = loop
        super().__init__(schema, keep_alive)

    def get_graphql_params(self, *args, **kwargs):
        params = super(AiohttpSubscriptionServer,
                       self).get_graphql_params(*args, **kwargs)
        return dict(params, return_promise=True,
                    executor=AsyncioExecutor(loop=self.loop))

    async def _handle(self, ws, request_context=None):
        connection_context = AiohttpConnectionContext(ws, request_context)
        await self.on_open(connection_context)
        pending = set()
        while True:
            try:
                if connection_context.closed:
                    raise ConnectionClosedException()
                message = await connection_context.receive()
            except ConnectionClosedException:
                break
            finally:
                if pending:
                    (_, pending) = await wait(pending, timeout=0, loop=self.loop)

            task = ensure_future(
                self.on_message(connection_context, message), loop=self.loop)
            pending.add(task)

        self.on_close(connection_context)
        for task in pending:
            task.cancel()

    async def handle(self, ws, request_context=None):
        await shield(self._handle(ws, request_context), loop=self.loop)

    async def on_open(self, connection_context):
        pass

    def on_close(self, connection_context):
        remove_operations = list(connection_context.operations.keys())
        for op_id in remove_operations:
            self.unsubscribe(connection_context, op_id)

    async def on_connect(self, connection_context, payload):
        pass

    async def on_connection_init(self, connection_context, op_id, payload):
        try:
            await self.on_connect(connection_context, payload)
            await self.send_message(connection_context, op_type=GQL_CONNECTION_ACK)
        except Exception as e:
            await self.send_error(connection_context, op_id, e, GQL_CONNECTION_ERROR)
            await connection_context.close(1011)

    async def on_start(self, connection_context, op_id, params):
        execution_result = self.execute(
            connection_context.request_context, params)

        if isawaitable(execution_result):
            execution_result = await execution_result

        if not hasattr(execution_result, '__aiter__'):
            await self.send_execution_result(
                connection_context, op_id, execution_result)
        else:
            iterator = await execution_result.__aiter__()
            connection_context.register_operation(op_id, iterator)
            async for single_result in iterator:
                if not connection_context.has_operation(op_id):
                    break
                await self.send_execution_result(
                    connection_context, op_id, single_result)
            await self.send_message(connection_context, op_id, GQL_COMPLETE)

    async def on_stop(self, connection_context, op_id):
        self.unsubscribe(connection_context, op_id)



import json
from collections import OrderedDict

from graphql import graphql, format_error

from .constants import (
    GQL_CONNECTION_INIT,
    GQL_CONNECTION_TERMINATE,
    GQL_START,
    GQL_STOP,
    GQL_ERROR,
    GQL_CONNECTION_ERROR,
    GQL_DATA
)


class ConnectionClosedException(Exception):
    pass


class BaseConnectionContext(object):
    def __init__(self, ws, request_context=None):
        self.ws = ws
        self.operations = {}
        self.request_context = request_context

    def has_operation(self, op_id):
        return op_id in self.operations

    def register_operation(self, op_id, async_iterator):
        self.operations[op_id] = async_iterator

    def get_operation(self, op_id):
        return self.operations[op_id]

    def remove_operation(self, op_id):
        del self.operations[op_id]

    def receive(self):
        raise NotImplementedError("receive method not implemented")

    def send(self, data):
        raise NotImplementedError("send method not implemented")

    @property
    def closed(self):
        raise NotImplementedError("closed property not implemented")

    def close(self, code):
        raise NotImplementedError("close method not implemented")


class BaseSubscriptionServer(object):

    def __init__(self, schema, keep_alive=True):
        self.schema = schema
        self.keep_alive = keep_alive

    def get_graphql_params(self, connection_context, payload):
        return {
            'request_string': payload.get('query'),
            'variable_values': payload.get('variables'),
            'operation_name': payload.get('operationName'),
            'context_value': payload.get('context'),
        }

    def build_message(self, id, op_type, payload):
        message = {}
        if id is not None:
            message['id'] = id
        if op_type is not None:
            message['type'] = op_type
        if payload is not None:
            message['payload'] = payload
        return message

    def process_message(self, connection_context, parsed_message):
        op_id = parsed_message.get('id')
        op_type = parsed_message.get('type')
        payload = parsed_message.get('payload')

        if op_type == GQL_CONNECTION_INIT:
            return self.on_connection_init(connection_context, op_id, payload)

        elif op_type == GQL_CONNECTION_TERMINATE:
            return self.on_connection_terminate(connection_context, op_id)

        elif op_type == GQL_START:
            assert isinstance(payload, dict), "The payload must be a dict"

            params = self.get_graphql_params(connection_context, payload)
            if not isinstance(params, dict):
                error = Exception(
                    "Invalid params returned from get_graphql_params!"
                    " Return values must be a dict.")
                return self.send_error(connection_context, op_id, error)

            # If we already have a subscription with this id, unsubscribe from
            # it first
            if connection_context.has_operation(op_id):
                self.unsubscribe(connection_context, op_id)

            return self.on_start(connection_context, op_id, params)

        elif op_type == GQL_STOP:
            return self.on_stop(connection_context, op_id)

        else:
            return self.send_error(connection_context, op_id, Exception(
                "Invalid message type: {}.".format(op_type)))

    def send_execution_result(self, connection_context, op_id, execution_result):
        result = self.execution_result_to_dict(execution_result)
        return self.send_message(connection_context, op_id, GQL_DATA, result)

    def execution_result_to_dict(self, execution_result):
        result = OrderedDict()
        if execution_result.data:
            result['data'] = execution_result.data
        if execution_result.errors:
            result['errors'] = [format_error(error)
                                for error in execution_result.errors]
        return result

    def send_message(self, connection_context, op_id=None, op_type=None, payload=None):
        message = self.build_message(op_id, op_type, payload)
        assert message, "You need to send at least one thing"
        json_message = json.dumps(message)
        return connection_context.send(json_message)

    def send_error(self, connection_context, op_id, error, error_type=None):
        if error_type is None:
            error_type = GQL_ERROR

        assert error_type in [GQL_CONNECTION_ERROR, GQL_ERROR], (
            'error_type should be one of the allowed error messages'
            ' GQL_CONNECTION_ERROR or GQL_ERROR'
        )

        error_payload = {
            'message': str(error)
        }

        return self.send_message(
            connection_context,
            op_id,
            error_type,
            error_payload
        )

    def unsubscribe(self, connection_context, op_id):
        if connection_context.has_operation(op_id):
            # Close async iterator
            connection_context.get_operation(op_id).dispose()
            # Close operation
            connection_context.remove_operation(op_id)
        self.on_operation_complete(connection_context, op_id)

    def on_operation_complete(self, connection_context, op_id):
        pass

    def on_connection_terminate(self, connection_context, op_id):
        return connection_context.close(1011)

    def execute(self, request_context, params):
        return graphql(
            self.schema, **dict(params, allow_subscriptions=True))

    def handle(self, ws, request_context=None):
        raise NotImplementedError("handle method not implemented")

    def on_message(self, connection_context, message):
        try:
            if not isinstance(message, dict):
                parsed_message = json.loads(message)
                assert isinstance(
                    parsed_message, dict), "Payload must be an object."
            else:
                parsed_message = message
        except Exception as e:
            return self.send_error(connection_context, None, e)

        return self.process_message(connection_context, parsed_message)

    def on_open(self, connection_context):
        raise NotImplementedError("on_open method not implemented")

    def on_connect(self, connection_context, payload):
        raise NotImplementedError("on_connect method not implemented")

    def on_close(self, connection_context):
        raise NotImplementedError("on_close method not implemented")

    def on_connection_init(self, connection_context, op_id, payload):
        raise NotImplementedError("on_connection_init method not implemented")

    def on_stop(self, connection_context, op_id):
        raise NotImplementedError("on_stop method not implemented")

    def on_start(self, connection_context, op_id, params):
        raise NotImplementedError("on_start method not implemented")



import mock
from django.conf import settings

settings.configure()  # noqa

from graphql_ws.django_channels import (
    DjangoChannelConnectionContext,
    DjangoChannelSubscriptionServer,
)


class TestConnectionContext:
    def test_send(self):
        msg = mock.Mock()
        connection_context = DjangoChannelConnectionContext(message=msg)
        connection_context.send("test")
        msg.reply_channel.send.assert_called_with("test")

    def test_close(self):
        msg = mock.Mock()
        connection_context = DjangoChannelConnectionContext(message=msg)
        connection_context.close(123)
        msg.reply_channel.send.assert_called_with({"close": True, "text": 123})


def test_subscription_server_smoke():
    DjangoChannelSubscriptionServer(schema=None)



import pickle

import gevent
import redis

from rx.subjects import Subject
from rx import config


class SubjectObserversWrapper(object):
    def __init__(self, pubsub, channel):
        self.pubsub = pubsub
        self.channel = channel
        self.observers = []

        self.lock = config["concurrency"].RLock()

    def __getitem__(self, key):
        return self.observers[key]

    def __getattr__(self, attr):
        return getattr(self.observers, attr)

    def remove(self, observer):
        with self.lock:
            self.observers.remove(observer)
            if not self.observers:
                self.pubsub.unsubscribe(self.channel)


class GeventRxPubsub(object):

    def __init__(self):
        self.subscriptions = {}

    def publish(self, channel, payload):
        if channel in self.subscriptions:
            self.subscriptions[channel].on_next(payload)

    def subscribe_to_channel(self, channel):
        if channel in self.subscriptions:
            return self.subscriptions[channel]
        else:
            subject = Subject()
            # monkeypatch Subject to unsubscribe pubsub on observable
            # subscription.dispose()
            subject.observers = SubjectObserversWrapper(self, channel)
            self.subscriptions[channel] = subject
            return subject

    def unsubscribe(self, channel):
        if channel in self.subscriptions:
            del self.subscriptions[channel]


class GeventRxRedisPubsub(object):

    def __init__(self, host='localhost', port=6379, *args, **kwargs):
        redis.connection.socket = gevent.socket
        self.redis = redis.StrictRedis(host, port, *args, **kwargs)
        self.pubsub = self.redis.pubsub(ignore_subscribe_messages=True)
        self.subscriptions = {}
        self.greenlet = None

    def publish(self, channel, payload):
        self.redis.publish(channel, pickle.dumps(payload))

    def subscribe_to_channel(self, channel):
        if channel in self.subscriptions:
            return self.subscriptions[channel]
        else:
            self.pubsub.subscribe(channel)
            subject = Subject()
            # monkeypatch Subject to unsubscribe pubsub on observable
            # subscription.dispose()
            subject.observers = SubjectObserversWrapper(self, channel)
            self.subscriptions[channel] = subject
            if not self.greenlet:
                self.greenlet = gevent.spawn(self._wait_and_get_messages)
            return subject

    def unsubscribe(self, channel):
        if channel in self.subscriptions:
            self.pubsub.unsubscribe(channel)
            del self.subscriptions[channel]
        if not self.subscriptions:
            self.greenlet.kill()

    def _wait_and_get_messages(self):
        while True:
            msg = self.pubsub.get_message()
            if msg:
                if isinstance(msg['channel'], bytes):
                    channel = msg['channel'].decode()
                else:
                    channel = msg['channel']
                if channel in self.subscriptions:
                    self.subscriptions[channel].on_next(pickle.loads(
                        msg['data']))
            gevent.sleep(.001)



# -*- coding: utf-8 -*-

"""Top-level package for GraphQL WS."""

__author__ = """Syrus Akbary"""
__email__ = 'me@syrusakbary.com'
__version__ = '0.0.1'


from .base import BaseConnectionContext, BaseSubscriptionServer  # noqa: F401


from unittest import mock

import pytest
from aiohttp import WSMsgType

from graphql_ws.aiohttp import AiohttpConnectionContext, AiohttpSubscriptionServer
from graphql_ws.base import ConnectionClosedException


class AsyncMock(mock.Mock):
    def __call__(self, *args, **kwargs):

        async def coro():
            return super(AsyncMock, self).__call__(*args, **kwargs)

        return coro()


@pytest.fixture()
def mock_ws():
    ws = AsyncMock(spec=["receive", "send_str", "closed", "close"])
    ws.closed = False
    ws.receive.return_value = AsyncMock(spec=["type", "data"])
    return ws


@pytest.mark.asyncio
class TestConnectionContext:
    async def test_receive_good_data(self, mock_ws):
        msg = mock_ws.receive.return_value
        msg.type = WSMsgType.TEXT
        msg.data = "test"
        connection_context = AiohttpConnectionContext(ws=mock_ws)
        assert await connection_context.receive() == "test"

    async def test_receive_error(self, mock_ws):
        msg = mock_ws.receive.return_value
        msg.type = WSMsgType.ERROR
        connection_context = AiohttpConnectionContext(ws=mock_ws)
        with pytest.raises(ConnectionClosedException):
            await connection_context.receive()

    async def test_receive_closing(self, mock_ws):
        mock_ws.receive.return_value.type = WSMsgType.CLOSING
        connection_context = AiohttpConnectionContext(ws=mock_ws)
        with pytest.raises(ConnectionClosedException):
            await connection_context.receive()

    async def test_receive_closed(self, mock_ws):
        mock_ws.receive.return_value.type = WSMsgType.CLOSED
        connection_context = AiohttpConnectionContext(ws=mock_ws)
        with pytest.raises(ConnectionClosedException):
            await connection_context.receive()

    async def test_send(self, mock_ws):
        connection_context = AiohttpConnectionContext(ws=mock_ws)
        await connection_context.send("test")
        mock_ws.send_str.assert_called_with("test")

    async def test_send_closed(self, mock_ws):
        mock_ws.closed = True
        connection_context = AiohttpConnectionContext(ws=mock_ws)
        await connection_context.send("test")
        mock_ws.send_str.assert_not_called()

    async def test_close(self, mock_ws):
        connection_context = AiohttpConnectionContext(ws=mock_ws)
        await connection_context.close(123)
        mock_ws.close.assert_called_with(code=123)


def test_subscription_server_smoke():
    AiohttpSubscriptionServer(schema=None)
